@@ -26,7 +26,6 @@ def test_missing_command(mockexecvpe):
 
 
 def test_command_not_found(monkeypatch):
-
     dummy_error_msg = (
         "This is dummy error message, file not found or something like that"
     )
@@ -39,7 +38,7 @@ def test_command_not_found(monkeypatch):
 
     assert xexec([command]) == (
         None,
-        "xonsh: exec: file not found: {}: {}" "\n".format(dummy_error_msg, command),
+        f"xonsh: exec: file not found: {dummy_error_msg}: {command}" "\n",
         1,
     )
 
