@@ -12,30 +12,30 @@ def parser():
 @pytest.mark.parametrize(
     "cmd",
     (
-        "ls $WO",
-        "ls /home/$WO",
-        "ls '/home/$WO'",
-        "ls @('hi ' + $WO",
+        "ls $WOW",
+        "ls /home/$WOW",
+        "ls '/home/$WOW'",
+        "ls @('hi ' + $WOW",
     ),
 )
 def test_simple(cmd, xession, monkeypatch, parser):
-    xession.env.update({"WOW": 1})
+    xession.env.update({"WOWZER": 1})
 
     context = parser.parse(cmd, len(cmd))
     comps, lprefix = complete_environment_vars(context)
     # account for the ending quote
     if cmd[-1] in "'":
-        assert lprefix == 4
+        assert lprefix == 5
     else:
-        assert lprefix == 3
-    assert set(comps) == {"$WOW"}
+        assert lprefix == 4
+    assert set(comps) == {"$WOWZER"}
 
 
 def test_rich_completions(xession, monkeypatch, parser):
-    xession.env.update({"WOW": 1})
-    xession.env.register("WOW", type=int, doc="Nice Docs!")
+    xession.env.update({"WOWZER": 1})
+    xession.env.register("WOWZER", type=int, doc="Nice Docs!")
 
-    context = parser.parse("$WO", 3)
+    context = parser.parse("$WOW", 4)
     completion = next(complete_environment_vars(context)[0])
-    assert completion.display == "$WOW [int]"
+    assert completion.display == "$WOWZER [int]"
     assert completion.description == "Nice Docs!"
