@@ -64,6 +64,7 @@ def run_xonsh(
     args=None,
     timeout=20,
     env=None,
+    blocking=True,
 ):
     # Env
     popen_env = dict(os.environ)
@@ -107,6 +108,9 @@ def run_xonsh(
         proc.stdin.write(stdin_cmd)
         proc.stdin.flush()
 
+    if not blocking:
+        return proc
+
     try:
         out, err = proc.communicate(input=input, timeout=timeout)
     except sp.TimeoutExpired:
@@ -1356,6 +1360,58 @@ def test_catching_exit_signal():
     assert ret > 0
 
 
+@skip_if_on_windows
+def test_forwarding_sighup(tmpdir):
+    """We want to make sure that SIGHUP is forwarded to subprocesses when
+    received, so we spin up a Bash process that waits for SIGHUP and then
+    writes `SIGHUP` to a file, then exits. Then we check the content of
+    that file to ensure that the Bash process really did get SIGHUP."""
+    outfile = tmpdir.mkdir("xonsh_test_dir").join("sighup_test.out")
+
+    stdin_cmd = f"""
+sleep 0.2
+(sleep 1 && kill -SIGHUP @(__import__('os').getppid())) &
+bash -c "trap 'echo SIGHUP > {outfile}; exit 0' HUP; sleep 30 & wait $!"
+"""
+    proc = run_xonsh(
+        cmd=None,
+        stdin_cmd=stdin_cmd,
+        stderr=sp.PIPE,
+        interactive=True,
+        single_command=False,
+        blocking=False,
+    )
+    proc.wait(timeout=5)
+    # if this raises FileNotFoundError, then the Bash subprocess probably did not get SIGHUP
+    assert outfile.read_text("utf-8").strip() == "SIGHUP"
+
+
+@skip_if_on_windows
+def test_on_postcommand_waiting(tmpdir):
+    """Ensure that running a subcommand in the on_postcommand hook doesn't
+    block xonsh from exiting when there is a running foreground process."""
+    outdir = tmpdir.mkdir("xonsh_test_dir")
+
+    stdin_cmd = f"""
+sleep 0.2
+@events.on_postcommand
+def postcmd_hook(**kwargs):
+    touch {outdir}/sighup_test_postcommand
+
+(sleep 1 && kill -SIGHUP @(__import__('os').getppid())) &
+bash -c "trap '' HUP; sleep 30"
+"""
+    proc = run_xonsh(
+        cmd=None,
+        stdin_cmd=stdin_cmd,
+        stderr=sp.PIPE,
+        interactive=True,
+        single_command=False,
+        blocking=False,
+    )
+    proc.wait(timeout=5)
+
+
 @skip_if_on_windows
 def test_suspended_captured_process_pipeline():
     """See also test_specs.py:test_specs_with_suspended_captured_process_pipeline"""
@@ -1515,7 +1571,10 @@ def test_xonshrc(tmpdir, cmd, exp):
     (script_xsh := home / "script.xsh").write_text("echo SCRIPT_XSH", encoding="utf8")
 
     # Construct $XONSHRC and $XONSHRC_DIR.
-    xonshrc_files = [str(home_config_xonsh_rc_xsh), str(home_xonsh_rc_path)]
+    xonshrc_files = [
+        str(home_config_xonsh_rc_xsh),
+        str(home_xonsh_rc_path),
+    ]
     xonshrc_dir = [str(home_config_xonsh_rcd)]
 
     args = [
@@ -1535,7 +1594,6 @@ def test_xonshrc(tmpdir, cmd, exp):
         env=env,
     )
 
-    exp = exp
     assert re.match(
         exp,
         out,
