@@ -1,15 +1,16 @@
 """Tests xonsh tools."""
+
 import datetime as dt
 import os
 import pathlib
-import stat
+import re
+import subprocess
 import warnings
-from tempfile import TemporaryDirectory
 
 import pytest
 
 from xonsh import __version__
-from xonsh.lexer import Lexer
+from xonsh.parsers.lexer import Lexer
 from xonsh.platform import HAS_PYGMENTS, ON_WINDOWS, PYTHON_VERSION_INFO
 from xonsh.pytest.tools import skip_if_on_windows
 from xonsh.tools import (
@@ -32,7 +33,6 @@ from xonsh.tools import (
     ensure_timestamp,
     env_path_to_str,
     escape_windows_cmd_string,
-    executables_in,
     expand_case_matching,
     expand_path,
     expandvars,
@@ -68,6 +68,7 @@ from xonsh.tools import (
     pathsep_to_seq,
     pathsep_to_set,
     pathsep_to_upper_seq,
+    print_exception,
     register_custom_style,
     replace_logical_line,
     seq_to_pathsep,
@@ -97,7 +98,6 @@ INDENT = "    "
 
 TOOLS_ENV = {"EXPAND_ENV_VARS": True, "XONSH_ENCODING_ERRORS": "strict"}
 ENCODE_ENV_ONLY = {"XONSH_ENCODING_ERRORS": "strict"}
-PATHEXT_ENV = {"PATHEXT": [".COM", ".EXE", ".BAT"]}
 
 
 def test_random_choice():
@@ -952,12 +952,12 @@ def test_env_path_add(left, right, exp):
 def test_env_path_add_replace_no_dupes_front_replace_existing():
     # Test replaces without dupes when added to front when adding existing entry
     path = EnvPath(
-        [os.pathsep.join(["home", "wakka"]), os.pathsep.join(["home", "wakka", "bin"])]
+        [os.sep.join(["home", "wakka"]), os.sep.join(["home", "wakka", "bin"])]
     )
-    path.add(os.pathsep.join(["home", "wakka", "bin"]), front=True, replace=True)
+    path.add(os.sep.join(["home", "wakka", "bin"]), front=True, replace=True)
     assert path == [
-        os.pathsep.join(["home", "wakka", "bin"]),
-        os.pathsep.join(["home", "wakka"]),
+        os.sep.join(["home", "wakka", "bin"]),
+        os.sep.join(["home", "wakka"]),
     ]
 
 
@@ -965,15 +965,15 @@ def test_env_path_add_replace_no_dupes_front_replace_multiple():
     # Test replaces without dupes when added to front when multiple existing occurrences
     path = EnvPath(
         [
-            os.pathsep.join(["home", "wakka"]),
-            os.pathsep.join(["home", "wakka", "bin"]),
-            os.pathsep.join(["home", "wakka", "bin"]),
+            os.sep.join(["home", "wakka"]),
+            os.sep.join(["home", "wakka", "bin"]),
+            os.sep.join(["home", "wakka", "bin"]),
         ]
     )
-    path.add(os.pathsep.join(["home", "wakka", "bin"]), front=True, replace=True)
+    path.add(os.sep.join(["home", "wakka", "bin"]), front=True, replace=True)
     assert path == [
-        os.pathsep.join(["home", "wakka", "bin"]),
-        os.pathsep.join(["home", "wakka"]),
+        os.sep.join(["home", "wakka", "bin"]),
+        os.sep.join(["home", "wakka"]),
     ]
 
 
@@ -981,38 +981,78 @@ def test_env_path_add_replace_no_dupes_back_replace_multiple():
     # Test replaces without dupes when not added to front
     path = EnvPath(
         [
-            os.pathsep.join(["home", "wakka"]),
-            os.pathsep.join(["home", "wakka", "bin"]),
-            os.pathsep.join(["home", "wakka", "bin"]),
+            os.sep.join(["home", "wakka"]),
+            os.sep.join(["home", "wakka", "bin"]),
+            os.sep.join(["home", "wakka", "bin"]),
         ]
     )
-    path.add(os.pathsep.join(["home", "wakka", "bin"]), front=False, replace=True)
+    path.add(os.sep.join(["home", "wakka", "bin"]), front=False, replace=True)
     assert path == [
-        os.pathsep.join(["home", "wakka"]),
-        os.pathsep.join(["home", "wakka", "bin"]),
+        os.sep.join(["home", "wakka"]),
+        os.sep.join(["home", "wakka", "bin"]),
     ]
 
 
 def test_env_path_add_pathlib():
-    os.pathsep.join(["home", "wakka", "bin"])
+    os.sep.join(["home", "wakka", "bin"])
     path = EnvPath(
         [
-            os.pathsep.join(["home", "wakka"]),
-            os.pathsep.join(["home", "wakka", "bin"]),
-            os.pathsep.join(["home", "wakka", "bin"]),
+            os.sep.join(["home", "wakka"]),
+            os.sep.join(["home", "wakka", "bin"]),
+            os.sep.join(["home", "wakka", "bin"]),
         ]
     )
     path.add(
-        pathlib.Path(os.pathsep.join(["home", "wakka", "bin"])),
+        pathlib.Path(os.sep.join(["home", "wakka", "bin"])),
         front=False,
         replace=True,
     )
     assert path == [
-        os.pathsep.join(["home", "wakka"]),
-        os.pathsep.join(["home", "wakka", "bin"]),
+        os.sep.join(["home", "wakka"]),
+        os.sep.join(["home", "wakka", "bin"]),
     ]
 
 
+def test_env_path_append_remove_pathlib_path():
+    path = EnvPath()
+
+    # Append-remove
+    path.append(os.sep.join(["home", "dino"]))
+    path.remove(os.sep.join(["home", "dino"]))
+
+    path.append(os.sep.join(["~", "dino"]))
+    path.remove(pathlib.Path(os.sep.join(["~", "dino"])))
+
+    path.append(pathlib.Path(os.sep.join(["~", "dino"])))
+    path.remove(pathlib.Path(os.sep.join(["~", "dino"])))
+
+    path.append(pathlib.Path(os.sep.join(["~", "dino"])))
+    path.remove(os.sep.join(["~", "dino"]))
+
+    path.append(
+        pathlib.Path(os.sep.join([str(pathlib.Path("~").expanduser()), "dino"]))
+    )
+    path.remove(os.sep.join(["~", "dino"]))
+
+    path.append(pathlib.Path(os.sep.join(["~", "dino"])))
+    path.remove(os.sep.join([str(pathlib.Path("~").expanduser()), "dino"]))
+
+    # Insert-remove
+    path.insert(0, os.sep.join(["home", "dino"]))
+    path.remove(os.sep.join(["home", "dino"]))
+
+    path.insert(0, os.sep.join(["~", "dino"]))
+    path.remove(pathlib.Path(os.sep.join(["~", "dino"])))
+
+    path.insert(0, pathlib.Path(os.sep.join(["~", "dino"])))
+    path.remove(pathlib.Path(os.sep.join(["~", "dino"])))
+
+    path.prepend(pathlib.Path(os.sep.join(["~", "dino"])))
+    path.remove(os.sep.join(["~", "dino"]))
+
+    assert path == []
+
+
 # helper
 def expand(path):
     return os.path.expanduser(os.path.expandvars(path))
@@ -1426,9 +1466,11 @@ def test_is_logfile_opt(inp, exp):
         pytest.param("/dev/null", "/dev/null", marks=skip_if_on_windows),
         pytest.param(
             "/dev/nonexistent_dev",
-            "/dev/nonexistent_dev"
-            if is_writable_file("/dev/nonexistent_dev")
-            else None,
+            (
+                "/dev/nonexistent_dev"
+                if is_writable_file("/dev/nonexistent_dev")
+                else None
+            ),
             marks=skip_if_on_windows,
         ),
         ("~/log", os.path.expanduser("~/log")),
@@ -1570,46 +1612,6 @@ def test_partial_string(leaders, prefix, quote):
     assert obs == exp
 
 
-def test_executables_in(xession):
-    expected = set()
-    types = ("file", "directory", "brokensymlink")
-    if ON_WINDOWS:
-        # Don't test symlinks on windows since it requires admin
-        types = ("file", "directory")
-    executables = (True, False)
-    with TemporaryDirectory() as test_path:
-        for _type in types:
-            for executable in executables:
-                fname = f"{_type}_{executable}"
-                if _type == "none":
-                    continue
-                if _type == "file" and executable:
-                    ext = ".exe" if ON_WINDOWS else ""
-                    expected.add(fname + ext)
-                else:
-                    ext = ""
-                path = os.path.join(test_path, fname + ext)
-                if _type == "file":
-                    with open(path, "w") as f:
-                        f.write(fname)
-                elif _type == "directory":
-                    os.mkdir(path)
-                elif _type == "brokensymlink":
-                    tmp_path = os.path.join(test_path, "i_wont_exist")
-                    with open(tmp_path, "w") as f:
-                        f.write("deleteme")
-                        os.symlink(tmp_path, path)
-                    os.remove(tmp_path)
-                if executable and not _type == "brokensymlink":
-                    os.chmod(path, stat.S_IXUSR | stat.S_IRUSR | stat.S_IWUSR)
-            if ON_WINDOWS:
-                xession.env = PATHEXT_ENV
-                result = set(executables_in(test_path))
-            else:
-                result = set(executables_in(test_path))
-    assert expected == result
-
-
 @pytest.mark.parametrize(
     "inp, exp",
     [
@@ -2048,3 +2050,45 @@ from xonsh.style_tools import Token
 )
 def test_is_tok_color_dict(val, exp):
     assert is_tok_color_dict(val) == exp
+
+
+def test_print_exception_msg(xession, capsys):
+    xession.env["COLOR_INPUT"] = False
+
+    try:
+        a = 1 / 0
+        a += 1
+    except ZeroDivisionError:
+        print_exception(msg="MSG")
+    cap = capsys.readouterr()
+    assert cap.err.endswith("MSG\n"), f"captured_stderr = {cap.captured_stderr!r}"
+
+
+def test_print_exception_error(xession, capsys):
+    xession.env["COLOR_INPUT"] = False
+
+    with xession.env.swap(XONSH_SHOW_TRACEBACK=False):
+        try:
+            raise subprocess.CalledProcessError(1, ["ls", "nofile"], output="nooutput")
+        except subprocess.CalledProcessError:
+            print_exception(msg="MSG")
+    cap = capsys.readouterr()
+    match = "subprocess.CalledProcessError: Command .* returned non-zero exit status .*\nMSG\n"
+    assert re.match(
+        match,
+        cap.err,
+        re.MULTILINE | re.DOTALL,
+    ), f"\nAssert: {cap.err!r},\nexpected: {match!r}"
+
+    with xession.env.swap(XONSH_SHOW_TRACEBACK=True):
+        try:
+            raise subprocess.CalledProcessError(1, ["ls", "nofile"], output="nooutput")
+        except subprocess.CalledProcessError:
+            print_exception(msg="MSG")
+    cap = capsys.readouterr()
+    match = ".*Traceback.*subprocess.CalledProcessError: Command .* returned non-zero exit status .*MSG\n"
+    assert re.match(
+        match,
+        cap.err,
+        re.MULTILINE | re.DOTALL,
+    ), f"\nAssert: {cap.err!r},\nexpected {match!r}"
