@@ -189,23 +189,11 @@ def add_args(
         kwargs.setdefault("help", doc.params.get(name))
 
         completer = kwargs.pop("completer", None)
-        cls_inst = getattr(func, "__self__", None)
-
-        if cls_inst and isinstance(cls_inst, ArgParserAlias):
-            flags, kwargs = cls_inst.hook_pre_add_argument(
-                param=name, func=func, flags=flags, kwargs=kwargs
-            )
-
         action = parser.add_argument(*flags, **kwargs)
 
         if completer:
             action.completer = completer  # type: ignore
 
-        if cls_inst and isinstance(cls_inst, ArgParserAlias):
-            cls_inst.hook_post_add_argument(
-                parser=parser, action=action, param=name, func=func
-            )
-
         action.help = action.help or ""
         # Don't show default when
         # 1. None : No value is given for the option
@@ -222,7 +210,7 @@ def add_args(
 
 def make_parser(
     func: tp.Union[tp.Callable, str],
-    empty_help=True,
+    empty_help=False,
     **kwargs,
 ) -> "ArgParser":
     """A bare-bones argparse builder from functions"""
@@ -234,9 +222,7 @@ def make_parser(
             kwargs["epilog"] = doc.epilog
     parser = ArgParser(**kwargs)
     if empty_help:
-        parser.set_defaults(
-            **{_FUNC_NAME: lambda stdout=None: parser.print_help(file=stdout)}
-        )
+        parser.default_command = "--help"
     return parser
 
 
@@ -317,9 +303,14 @@ class ArgParser(ap.ArgumentParser):
 
         super().__init__(**kwargs)
         self.commands = None
+        self.default_command = None
 
     def add_command(
-        self, func: tp.Callable, args: tp.Optional[tp.Iterable[str]] = None, **kwargs
+        self,
+        func: tp.Callable,
+        default=False,
+        args: "tuple[str, ...] | None" = None,
+        **kwargs,
     ):
         """
             create a sub-parser and call this function during dispatch
@@ -333,6 +324,8 @@ class ArgParser(ap.ArgumentParser):
             Use _parser to get the generated parser instance.
             Use _args to get what is passed from sys.argv
             Use _parsed to get result of ``parser.parse_args``
+        default
+            Marks this sub-command as the default command for this parser.
         args
             if given only add these arguments to the parser.
             Otherwise all parameters to the function without `_` prefixed
@@ -353,13 +346,30 @@ class ArgParser(ap.ArgumentParser):
         name = kwargs.pop("prog", None)
         if not name:
             name = func.__name__.lstrip("_").replace("_", "-")
+
+        if default:
+            self.default_command = name
+
         parser = self.commands.add_parser(name, **kwargs)
-        add_args(parser, func, allowed_params=args)
+        add_args(parser, func, allowed_params=args, doc=doc)
         return parser
 
+    def _parse_known_args(
+        self, arg_strings: list[str], namespace: ap.Namespace, *args, **kwargs
+    ):
+        arg_set = set(arg_strings)
+        if (
+            self.commands
+            and self.default_command
+            and ({"-h", "--help"}.isdisjoint(arg_set))
+            and (set(self.commands.choices).isdisjoint(arg_set))
+        ):
+            arg_strings = [self.default_command] + arg_strings
+        return super()._parse_known_args(arg_strings, namespace, *args, **kwargs)
 
-def _dispatch_func(func: tp.Callable, ns: dict[str, tp.Any]):
-    """Final dispatch to the function based on signature."""
+
+def run_with_partial_args(func: tp.Callable, ns: dict[str, tp.Any]):
+    """Run function based on signature. Filling the arguments will be based on the values in ``ns``."""
     sign = inspect.signature(func)
     kwargs = {}
     for name, param in sign.parameters.items():
@@ -387,6 +397,8 @@ def dispatch(parser: ap.ArgumentParser, args=None, lenient=False, **ns):
     ns
         a dict that will be passed to underlying function
     """
+    ns.setdefault("_parser", parser)
+    ns.setdefault("_args", args)
 
     if lenient:
         parsed, unparsed = parser.parse_known_args(args)
@@ -396,8 +408,9 @@ def dispatch(parser: ap.ArgumentParser, args=None, lenient=False, **ns):
     ns["_parsed"] = parsed
     ns.update(vars(parsed))
 
-    func = ns[_FUNC_NAME]
-    return _dispatch_func(func, ns)
+    if _FUNC_NAME in ns:
+        func = ns[_FUNC_NAME]
+        return run_with_partial_args(func, ns)
 
 
 class ArgparseCompleter:
@@ -532,9 +545,12 @@ class ArgparseCompleter:
             if not act_res:
                 # it is not a option string: pass
                 break
+            if isinstance(act_res, list):
+                assert len(act_res) == 1
+                act_res = act_res[0]
             # it is a valid option and advance
             self.remaining_args = self.remaining_args[1:]
-            act, _, value = act_res
+            act, *_, value = act_res
 
             # remove the found option
             # todo: not remove if append/extend
@@ -575,37 +591,21 @@ class ArgParserAlias:
         self.stdout = None
         self.stderr = None
 
-    def build(self):
+    def build(self) -> "ArgParser":
         """Sub-classes should return constructed ArgumentParser"""
         if self.kwargs:
             return self.create_parser(**self.kwargs)
         raise NotImplementedError
 
-    def hook_pre_add_argument(self, param: str, func, flags, kwargs):
-        """Hook to update arguments that are passed to parser.add_argument"""
-        return flags, kwargs
-
-    def hook_post_add_argument(
-        self,
-        *,
-        parser: "ArgParser|ap.ArgumentParser",
-        action: "ap.Action",
-        param: str,
-        func: "tp.Callable",
-    ):
-        """Hook into parser.add_argument step.
-
-        Can be used to update action's attributes
-        """
-        return
-
     @property
     def parser(self):
         if self._parser is None:
             self._parser = self.build()
         return self._parser
 
-    def create_parser(self, func=None, has_args=False, allowed_params=None, **kwargs):
+    def create_parser(
+        self, func=None, has_args=False, allowed_params=None, **kwargs
+    ) -> "ArgParser":
         """create root parser"""
         func = func or self
         has_args = has_args or bool(allowed_params)
@@ -651,8 +651,6 @@ class ArgParserAlias:
             result = dispatch(
                 self.parser,
                 args,
-                _parser=self.parser,
-                _args=args,
                 _stdin=stdin,
                 _stdout=stdout,
                 _stderr=stderr,
