@@ -1,11 +1,10 @@
 """This module provides the implementation for the retrieving completion results
 from bash.
 """
-import functools
 
 # developer note: this file should not perform any action on import.
-#                 This file comes from https://github.com/xonsh/py-bash-completion
-#                 and should be edited there!
+# This is to allow users who want to use this completion file as a standalone CLI tool.
+import functools
 import os
 import pathlib
 import platform
@@ -16,7 +15,7 @@ import subprocess
 import sys
 import typing as tp
 
-__version__ = "0.2.7"
+__version__ = "0.2.8"
 
 
 @functools.lru_cache(1)
@@ -43,7 +42,7 @@ def _windows_bash_command(env=None):
             out = subprocess.check_output(
                 [bash_on_path, "--version"],
                 stderr=subprocess.PIPE,
-                text=True,
+                universal_newlines=True,
             )
         except subprocess.CalledProcessError:
             bash_works = False
@@ -106,7 +105,7 @@ _BASH_COMPLETIONS_PATHS_DEFAULT: tuple[str, ...] = ()
 def _get_bash_completions_source(paths=None):
     global _BASH_COMPLETIONS_PATHS_DEFAULT
     if paths is None:
-        if _BASH_COMPLETIONS_PATHS_DEFAULT is None:
+        if not _BASH_COMPLETIONS_PATHS_DEFAULT:
             _BASH_COMPLETIONS_PATHS_DEFAULT = _bash_completion_paths_default()
         paths = _BASH_COMPLETIONS_PATHS_DEFAULT
     for path in map(pathlib.Path, paths):
@@ -191,7 +190,7 @@ def _bash_quote_paths(paths, start, end):
         else:
             _tail = ""
         if start != "" and "r" not in start and backslash in s:
-            start = "r%s" % start
+            start = f"r{start}"
         s = s + _tail
         if end != "":
             if "r" not in start.lower():
@@ -199,7 +198,7 @@ def _bash_quote_paths(paths, start, end):
             if s.endswith(backslash) and not s.endswith(double_backslash):
                 s += backslash
         if end in s:
-            s = s.replace(end, "".join("\\%s" % i for i in end))
+            s = s.replace(end, "".join(f"\\{i}" for i in end))
         out.add(start + s + end)
     return out, need_quotes
 
@@ -351,7 +350,6 @@ def bash_completions(
     cmd = splt[0]
     cmd = os.path.basename(cmd)
     prev = ""
-
     if arg_index is not None:
         n = arg_index
         if arg_index > 0:
@@ -365,10 +363,8 @@ def bash_completions(
                 if idx >= begidx:
                     break
             prev = tok
-
         if len(prefix) == 0:
             n += 1
-
     prefix_quoted = shlex.quote(prefix)
 
     script = BASH_COMPLETE_SCRIPT.format(
@@ -387,20 +383,21 @@ def bash_completions(
     try:
         out = subprocess.check_output(
             [command, "-c", script],
-            text=True,
+            universal_newlines=True,
             stderr=subprocess.PIPE,
             env=env,
         )
+        out = [line for line in out.splitlines() if line.strip()]
         if not out:
             raise ValueError
     except (
         subprocess.CalledProcessError,
         FileNotFoundError,
+        UnicodeDecodeError,
         ValueError,
     ):
         return set(), 0
 
-    out = out.splitlines()
     complete_stmt = out[0]
     out = set(out[1:])
 
