@@ -1,4 +1,5 @@
 """Implements the xonsh history backend via sqlite3."""
+
 import collections
 import json
 import os
@@ -135,7 +136,7 @@ def _xh_sqlite_insert_command(cursor, cmd, sessionid, store_stdout, remove_dupli
 
 
 def _xh_sqlite_get_count(cursor, sessionid=None):
-    sql = "SELECT count(*) FROM xonsh_history "
+    sql = f"SELECT count(*) FROM {XH_SQLITE_TABLE_NAME} "
     params = []
     if sessionid is not None:
         sql += "WHERE sessionid = ? "
@@ -145,7 +146,7 @@ def _xh_sqlite_get_count(cursor, sessionid=None):
 
 
 def _xh_sqlite_get_records(cursor, sessionid=None, limit=None, newest_first=False):
-    sql = "SELECT inp, tsb, rtn, frequency, cwd FROM xonsh_history "
+    sql = f"SELECT inp, tsb, rtn, frequency, cwd FROM {XH_SQLITE_TABLE_NAME} "
     params = []
     if sessionid is not None:
         sql += "WHERE sessionid = ? "
@@ -161,14 +162,14 @@ def _xh_sqlite_get_records(cursor, sessionid=None, limit=None, newest_first=Fals
 
 def _xh_sqlite_delete_records(cursor, size_to_keep):
     sql = "SELECT min(tsb) FROM ("
-    sql += "SELECT tsb FROM xonsh_history ORDER BY tsb DESC "
+    sql += f"SELECT tsb FROM {XH_SQLITE_TABLE_NAME} ORDER BY tsb DESC "
     sql += "LIMIT %d)" % size_to_keep
     cursor.execute(sql)
     result = cursor.fetchone()
     if not result:
         return
     max_tsb = result[0]
-    sql = "DELETE FROM xonsh_history WHERE tsb < ?"
+    sql = f"DELETE FROM {XH_SQLITE_TABLE_NAME} WHERE tsb < ?"
     result = cursor.execute(sql, (max_tsb,))
     return result.rowcount
 
@@ -203,9 +204,24 @@ def xh_sqlite_delete_items(size_to_keep, filename=None):
         return _xh_sqlite_delete_records(c, size_to_keep)
 
 
-def xh_sqlite_pull(filename, last_pull_time, current_sessionid):
-    sql = "SELECT inp FROM xonsh_history WHERE tsb > ? AND sessionid != ? ORDER BY tsb"
-    params = [last_pull_time, current_sessionid]
+def xh_sqlite_pull(filename, last_pull_time, current_sessionid, src_sessionid=None):
+    # ensure we don't duplicate history entries if some crazy person passes the current session
+    if src_sessionid == current_sessionid:
+        return []
+
+    if src_sessionid:
+        sql = (
+            f"SELECT inp FROM {XH_SQLITE_TABLE_NAME} WHERE tsb > ?"
+            " AND sessionid = ? ORDER BY tsb"
+        )
+        params = [last_pull_time, src_sessionid]
+    else:
+        sql = (
+            f"SELECT inp FROM {XH_SQLITE_TABLE_NAME} WHERE tsb > ?"
+            " AND sessionid != ? ORDER BY tsb"
+        )
+        params = [last_pull_time, current_sessionid]
+
     with _xh_sqlite_get_conn(filename=filename) as conn:
         c = conn.cursor()
         c.execute(sql, tuple(params))
@@ -214,7 +230,7 @@ def xh_sqlite_pull(filename, last_pull_time, current_sessionid):
 
 def xh_sqlite_wipe_session(sessionid=None, filename=None):
     """Wipe the current session's entries from the database."""
-    sql = "DELETE FROM xonsh_history WHERE sessionid = ?"
+    sql = f"DELETE FROM {XH_SQLITE_TABLE_NAME} WHERE sessionid = ?"
     with _xh_sqlite_get_conn(filename=filename) as conn:
         c = conn.cursor()
         _xh_sqlite_create_history_table(c)
@@ -226,10 +242,13 @@ def xh_sqlite_delete_input_matching(pattern, filename=None):
     with _xh_sqlite_get_conn(filename=filename) as conn:
         c = conn.cursor()
         _xh_sqlite_create_history_table(c)
+        deleted = 0
         for inp, *_ in _xh_sqlite_get_records(c):
             if pattern.match(inp):
-                sql = f"DELETE FROM xonsh_history WHERE inp = '{inp}'"
-                c.execute(sql)
+                sql = f"DELETE FROM {XH_SQLITE_TABLE_NAME} WHERE inp=?"
+                c.execute(sql, (inp,))
+                deleted += 1
+        return deleted
 
 
 class SqliteHistoryGC(threading.Thread):
@@ -365,23 +384,26 @@ class SqliteHistory(History):
         data["gc options"] = envs.get("XONSH_HISTORY_SIZE")
         return data
 
-    def pull(self, show_commands=False):
+    def pull(self, show_commands=False, src_sessionid=None):
         if not hasattr(XSH.shell.shell, "prompter"):
             print(f"Shell type {XSH.shell.shell} is not supported.")
             return 0
 
         cnt = 0
+        prev = None
         for r in xh_sqlite_pull(
-            self.filename, self.last_pull_time, str(self.sessionid)
+            self.filename, self.last_pull_time, str(self.sessionid), src_sessionid
         ):
             if show_commands:
                 print(r[0])
-            XSH.shell.shell.prompter.history.append_string(r[0])
-            cnt += 1
+            if r[0] != prev:
+                XSH.shell.shell.prompter.history.append_string(r[0])
+                cnt += 1
+            prev = r[0]
         self.last_pull_time = time.time()
         return cnt
 
-    def run_gc(self, size=None, blocking=True):
+    def run_gc(self, size=None, blocking=True, **_):
         self.gc = SqliteHistoryGC(wait_for_shell=False, size=size)
         if blocking:
             while self.gc.is_alive():
@@ -400,6 +422,6 @@ class SqliteHistory(History):
 
     def delete(self, pattern):
         """Deletes all entries in the database where the input matches a pattern."""
-        xh_sqlite_delete_input_matching(
+        return xh_sqlite_delete_input_matching(
             pattern=re.compile(pattern), filename=self.filename
         )
