@@ -1,6 +1,7 @@
 """Implements the xonsh (tab-)completion context parser.
 This parser is meant to parse a (possibly incomplete) command line.
 """
+
 import enum
 import os
 import re
@@ -16,10 +17,10 @@ from typing import (
     overload,
 )
 
-from xonsh.lazyasd import lazyobject
-from xonsh.lexer import Lexer
+from xonsh.lib.lazyasd import lazyobject
 from xonsh.parsers.base import Location, raise_parse_error
-from xonsh.ply.ply import yacc
+from xonsh.parsers.lexer import Lexer
+from xonsh.parsers.ply import yacc
 from xonsh.tools import check_for_partial_string, get_line_continuation
 
 
@@ -208,8 +209,7 @@ class Spanned(Generic[T]):
         expansion_obj: Union[
             "ExpandableObject", ExpansionOperation, Missing
         ] = Missing.MISSING,
-    ) -> "Spanned[T]":
-        ...
+    ) -> "Spanned[T]": ...
 
     @overload
     def replace(
@@ -222,8 +222,7 @@ class Spanned(Generic[T]):
         expansion_obj: Union[
             "ExpandableObject", ExpansionOperation, Missing
         ] = Missing.MISSING,
-    ) -> "Spanned[T2]":
-        ...
+    ) -> "Spanned[T2]": ...
 
     def replace(
         self,
@@ -331,7 +330,8 @@ class CompletionContextParser:
         "LT",
         "GT",
         "RSHIFT",
-        "IOREDIRECT",
+        "IOREDIRECT1",
+        "IOREDIRECT2",
     }
     used_tokens |= io_redir_tokens
     artificial_tokens = {"ANY"}
@@ -351,7 +351,7 @@ class CompletionContextParser:
 
         self.error = None
         self.debug = debug
-        self.lexer = Lexer(tolerant=True)
+        self.lexer = Lexer(tolerant=True, pymode=False)
         self.tokens = tuple(self.used_tokens | self.artificial_tokens)
 
         yacc_kwargs = dict(
@@ -464,8 +464,7 @@ class CompletionContextParser:
                 if self.cursor_in_span(inner_span) or (
                     # the cursor is in a space-separated multi keyword.
                     # even if the cursor's at the edge, the keyword should be considered as a normal arg
-                    tok.value in ("and", "or")
-                    and self.cursor_in_span(outer_span)
+                    tok.value in ("and", "or") and self.cursor_in_span(outer_span)
                 ):
                     tok.type = "ANY"
 
@@ -934,7 +933,9 @@ class CompletionContextParser:
             sub_expr = cast(ArgContext, arg.expansion_obj)
 
             # this arg is a subcommand or multiple subcommands, e.g. `$(a && b)`
-            expanded_obj: Optional[ArgContext] = self.try_expand_span(sub_expr, new_span)  # type: ignore
+            expanded_obj: Optional[ArgContext] = self.try_expand_span(  # type: ignore
+                sub_expr, new_span
+            )
             if expanded_obj is None:
                 return None
             return self.sub_expression_arg(expanded_obj)
@@ -964,7 +965,9 @@ class CompletionContextParser:
             # the last command is expandable
             # if it were an `ExpansionOperation`, `try_expand` would caught it instead
             expandable = cast(ExpandableObject, python_context.expansion_obj)
-            expanded_command: Optional[ExpandableObject] = self.try_expand_right(expandable, new_span.stop)  # type: ignore
+            expanded_command: Optional[ExpandableObject] = self.try_expand_right(
+                expandable, new_span.stop
+            )  # type: ignore
 
             if (
                 expanded_command is not None
