@@ -2,6 +2,7 @@
 compatibility layers to make use of the 'best' implementation available
 on a platform.
 """
+
 import collections.abc as cabc
 import ctypes  # noqa
 import functools
@@ -13,7 +14,7 @@ import signal
 import subprocess
 import sys
 
-from xonsh.lazyasd import LazyBool, lazybool, lazyobject
+from xonsh.lib.lazyasd import LazyBool, lazybool, lazyobject
 
 # do not import any xonsh-modules here to avoid circular dependencies
 
@@ -474,10 +475,11 @@ def os_environ():
         return os.environ
 
 
-@functools.lru_cache(1)
 def bash_command():
     """Determines the command for Bash on the current platform."""
-    if ON_WINDOWS:
+    if (bc := os.getenv("XONSH_BASH_PATH_OVERRIDE", None)) is not None:
+        bc = str(bc)  # for pathlib Paths
+    elif ON_WINDOWS:
         bc = windows_bash_command()
     else:
         bc = "bash"
@@ -496,6 +498,7 @@ def BASH_COMPLETIONS_DEFAULT():
             "/usr/local/share/bash-completion/bash_completion",  # v2.x
             "/usr/local/etc/bash_completion",  # v1.x
             "/opt/homebrew/share/bash-completion/bash_completion",  # v2.x on M1
+            "/opt/homebrew/etc/bash_completion",  # v1.x on M1
         )
     elif ON_WINDOWS and git_for_windows_path():
         bcd = (
