@@ -1,26 +1,26 @@
 """Interface for running subprocess-mode commands on posix systems."""
-import os
-import io
-import sys
-import time
-import array
-import signal
-import threading
-import subprocess
 
-import xonsh.lazyasd as xl
+import array
+import io
+import os
+import signal
+import subprocess
+import sys
+import threading
+import time
+
+import xonsh.lib.lazyasd as xl
+import xonsh.lib.lazyimps as xli
 import xonsh.platform as xp
 import xonsh.tools as xt
-import xonsh.lazyimps as xli
 from xonsh.built_ins import XSH
-
+from xonsh.procs.jobs import proc_untraced_waitpid
 from xonsh.procs.readers import (
     BufferedFDParallelReader,
     NonBlockingFDReader,
     safe_fdclose,
 )
 
-
 # The following escape codes are xterm codes.
 # See http://rtfm.etla.org/xterm/ctlseq.html for more.
 MODE_NUMS = ("1049", "47", "1047")
@@ -28,12 +28,12 @@ MODE_NUMS = ("1049", "47", "1047")
 
 @xl.lazyobject
 def START_ALTERNATE_MODE():
-    return frozenset("\x1b[?{0}h".format(i).encode() for i in MODE_NUMS)
+    return frozenset(f"\x1b[?{i}h".encode() for i in MODE_NUMS)
 
 
 @xl.lazyobject
 def END_ALTERNATE_MODE():
-    return frozenset("\x1b[?{0}l".format(i).encode() for i in MODE_NUMS)
+    return frozenset(f"\x1b[?{i}l".encode() for i in MODE_NUMS)
 
 
 @xl.lazyobject
@@ -104,6 +104,14 @@ class PopenThread(threading.Thread):
             raise
 
         self.pid = proc.pid
+        self.name = repr(
+            {
+                "cls": self.__class__.__name__,
+                "name": self.name,
+                "cmd": args,
+                "pid": self.pid,
+            }
+        )
         self.universal_newlines = uninew = proc.universal_newlines
         if uninew:
             self.encoding = enc = env.get("XONSH_ENCODING")
@@ -161,6 +169,16 @@ class PopenThread(threading.Thread):
         # loop over reads while process is running.
         i = j = cnt = 1
         while proc.poll() is None:
+            info = proc_untraced_waitpid(proc, hang=False)
+            if getattr(proc, "suspended", False):
+                self.suspended = True
+                if XSH.env.get("XONSH_DEBUG", False):
+                    procname = f"{getattr(proc, 'args', '')} {proc.pid}".strip()
+                    print(
+                        f"Process {procname} suspended with signal {info['signal_name']}.",
+                        file=sys.stderr,
+                    )
+
             # this is here for CPU performance reasons.
             if i + j == 0:
                 cnt = min(cnt + 1, 1000)
@@ -358,9 +376,9 @@ class PopenThread(threading.Thread):
             return
         try:
             mode = xli.termios.tcgetattr(0)  # only makes sense for stdin
-            mode[xp.CC][
-                xli.termios.VSUSP
-            ] = self._tc_cc_vsusp  # set ^Z (ie SIGSTOP) to original
+            mode[xp.CC][xli.termios.VSUSP] = (
+                self._tc_cc_vsusp
+            )  # set ^Z (ie SIGSTOP) to original
             # this usually doesn't work in interactive mode,
             # but we should try it anyway.
             xli.termios.tcsetattr(0, xli.termios.TCSANOW, mode)
