@@ -1,4 +1,5 @@
 """Implements a xonsh tracer."""
+
 import importlib
 import inspect
 import linecache
@@ -10,9 +11,9 @@ import typing as tp
 import xonsh.procs.pipelines as xpp
 import xonsh.prompt.cwd as prompt
 from xonsh.cli_utils import Annotated, Arg, ArgParserAlias
-from xonsh.inspectors import find_file
-from xonsh.lazyasd import LazyObject
-from xonsh.lazyimps import pyghooks, pygments
+from xonsh.lib.inspectors import find_file
+from xonsh.lib.lazyasd import LazyObject
+from xonsh.lib.lazyimps import pyghooks, pygments
 from xonsh.platform import HAS_PYGMENTS
 from xonsh.tools import DefaultNotGiven, normabspath, print_color, to_bool
 
@@ -80,7 +81,7 @@ class TracerType:
                     frame.f_trace = self.prev_tracer
             self.prev_tracer = DefaultNotGiven
 
-    def trace(self, frame, event, arg):
+    def trace(self, frame, event, arg, *, find_file=find_file, print_color=print_color):
         """Implements a line tracing function."""
         if event not in self.valid_events:
             return self.trace
@@ -215,7 +216,7 @@ class TracerAlias(ArgParserAlias):
     """Tool for tracing xonsh code as it runs."""
 
     def build(self):
-        parser = self.create_parser(prog="trace")
+        parser = self.create_parser(prog="trace", empty_help=True)
         parser.add_command(tracer.on_files, prog="on", aliases=["start", "add"])
         parser.add_command(tracer.off_files, prog="off", aliases=["stop", "del", "rm"])
         parser.add_command(tracer.toggle_color, prog="color", aliases=["ls"])
