@@ -1,4 +1,5 @@
 """Implements the which xoreutil."""
+
 import argparse
 import functools
 import os
@@ -28,7 +29,7 @@ def _which_create_parser():
         "-s",
         "--skip-alias",
         action="store_true",
-        help="Do not search inxonsh.aliases",
+        help="Do not search in xonsh.aliases",
         dest="skip",
     )
     parser.add_argument(
@@ -161,11 +162,15 @@ def which(args, stdin=None, stdout=None, stderr=None, spec=None):
         original_os_path = xp.os_environ["PATH"]
         xp.os_environ["PATH"] = XSH.env.detype()["PATH"]
         matches = _which.whichgen(arg, exts=exts, verbose=verbose)
-        for abs_name, from_where in matches:
-            print_path(abs_name, from_where, stdout, verbose, captured)
-            nmatches += 1
-            if not pargs.all:
-                break
+        if matches is not None:
+            for match in matches:
+                if match is None:
+                    continue
+                abs_name, from_where = match
+                print_path(abs_name, from_where, stdout, verbose, captured)
+                nmatches += 1
+                if not pargs.all:
+                    break
         xp.os_environ["PATH"] = original_os_path
         if not nmatches:
             failures.append(arg)
